@@ -1739,14 +1739,8 @@ void 	save_channels (int servref)
 		if (tmp->server != servref)
 			continue;
 
-#if 0
-		/* You know, i'd be willing to bet this isnt neccesary */
-		if (!tmp->current_channel || !tmp->bind_channel)
-			continue;
-#else
 		if (tmp->waiting_channel)
 			continue;		/* Yea yea yea */
-#endif
 
 		tmp->waiting_channel = tmp->current_channel;
 		tmp->current_channel = NULL;
@@ -1760,6 +1754,21 @@ void 	save_channels (int servref)
 		tmpc->inactive = 1;
 }
 
+/* I don't know if this belongs here. */
+void	cant_join_channel (const char *channel, int server)
+{
+	Window *w = NULL;
+	while (traverse_all_windows(&w))
+	{
+		if (w->server != server)
+			continue;
+		if (!w->waiting_channel)
+			continue;
+		if (!my_stricmp(w->waiting_channel, channel))
+			new_free(&w->waiting_channel);
+	}
+	update_all_windows();
+}
 
 /**************************************************************************/
 /* 
