@@ -10,7 +10,7 @@
  */
 
 #if 0
-static	char	rcsid[] = "$Id: numbers.c,v 1.6 2001/01/12 20:11:36 jnelson Exp $";
+static	char	rcsid[] = "$Id: numbers.c,v 1.13 2001/03/24 03:43:40 jnelson Exp $";
 #endif
 
 #include "irc.h"
@@ -161,7 +161,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	user = (*ArgList[0]) ? ArgList[0] : NULL;
 
 	lastlog_level = set_lastlog_msg_level(LOG_CRAP);
-	message_from((char *) 0, LOG_CRAP);
+	message_from(NULL, LOG_CRAP);
 	ArgList++;
 	current_numeric = -comm;	/* must be negative of numeric! */
 
@@ -214,13 +214,14 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		new_port = atoi(ArgList[1]);
 		PasteArgs(ArgList, 2);
 
-		if (do_hook(current_numeric, "%s %s %d %s", from, 
-				new_server, new_port, ArgList[2]))
-			display_msg(from, ArgList);
-
+		/* Must do these things before calling "display_msg" */
 		add_to_server_list(new_server, new_port, NULL, NULL, 0);
 		server_reconnects_to(old_server, from_server);
 		from_server = old_server;
+
+		if (do_hook(current_numeric, "%s %s %d %s", from, 
+				new_server, new_port, ArgList[2]))
+			display_msg(from, ArgList);
 		break;
 	}
 
@@ -554,15 +555,15 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	case 402:
 	{
 		PasteArgs(ArgList, 1);
-		if (do_hook(current_numeric, "%s %s %s", from, 
-					ArgList[0], ArgList[1]))
-			display_msg(from, ArgList);
-
 		/* 
 		 * Some broken servers send this instead of a 315 numeric
 		 * when a who request has been completed.
 		 */
 		fake_who_end(from, ArgList[0]);
+
+		if (do_hook(current_numeric, "%s %s %s", from, 
+					ArgList[0], ArgList[1]))
+			display_msg(from, ArgList);
 		break;
 	}
 
@@ -573,15 +574,15 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	case 523:
 	{
 		PasteArgs(ArgList, 0);
-		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
-			display_msg(from, ArgList);
-
 		/* 
 		 * This dalnet error message doesn't even give us the
 		 * courtesy of telling us which who request was in error,
 		 * so we have to guess.  Whee.
 		 */
 		fake_who_end(from, NULL);
+
+		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+			display_msg(from, ArgList);
 		break;
 	}
 
@@ -605,16 +606,12 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 			break;
 		}
 
-		if (do_hook(current_numeric, "%s %s %s", from, 
-					ArgList[0], ArgList[1]))
-			display_msg(from, ArgList);
-
 		/* 
 		 * Some broken servers send this instead of a 315 numeric
 		 * when a who request has been completed.
 		 */
 		if (fake_who_end(from, ArgList[0]))
-			break;
+			;
 
 		/*
 		 * If you try to JOIN or PART the "*" named channel, as may
@@ -625,9 +622,13 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 * this server claims does not exist; we blow the channel away
 		 * for good measure.
 		 */
-		if (strcmp(ArgList[0], "*"))
+		else if (strcmp(ArgList[0], "*"))
 			remove_channel(ArgList[0], from_server);
 
+		if (do_hook(current_numeric, "%s %s %s", from, 
+					ArgList[0], ArgList[1]))
+			display_msg(from, ArgList);
+
 		break;
 	}
 
@@ -702,9 +703,9 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 */
 		if (ArgList[0] && ArgList[1] == NULL)
 		{
+			accept_server_nickname(from_server, user);
 			if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 				display_msg(from, ArgList);
-			accept_server_nickname(from_server, user);
 			break;
 		}
 
@@ -732,9 +733,9 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 */
 		if (is_server_connected(from_server))
 		{
+			accept_server_nickname(from_server, user);
 			if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 				display_msg(from, ArgList);
-			accept_server_nickname(from_server, user);
 			break;
 		}
 
@@ -767,9 +768,9 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 
 	case 439:		/* Comstud's "Can't change nickname" */
 	{
+		accept_server_nickname(from_server, user);
 		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 			display_msg(from, ArgList);
-		accept_server_nickname(from_server, user);
 		break;
 	}
 
@@ -819,13 +820,8 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	case 464:		/* #define ERR_PASSWDMISMATCH   464 */
 	{
 		PasteArgs(ArgList, 0);
-		if (!is_server_connected(from_server))
-		{
-			say("Odd server stuff from %s: %s", from, ArgList[0]);
-			break;
-		}
-
 		flag = do_hook(current_numeric, "%s %s", from, ArgList[0]);
+
 		if (oper_command)
 		{
 			if (flag)
@@ -849,6 +845,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		}
 		break;
 	}
+
 	case 465:		/* #define ERR_YOUREBANNEDCREEP 465 */
 	{
 		PasteArgs(ArgList, 0);
@@ -857,9 +854,9 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 * connected to a server, then doing say() is not
 		 * a good idea.  So now it just doesnt do anything.
 		 */
+		server_reconnects_to(from_server, -1);
 		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 			display_msg(from, ArgList);
-		server_reconnects_to(from_server, -1);
 		break;
 	}
 
@@ -879,6 +876,50 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	case 471:		/* #define ERR_CHANNELISFULL    471 */
+	case 473:		/* #define ERR_INVITEONLYCHAN   473 */
+	case 474:		/* #define ERR_BANNEDFROMCHAN   474 */
+	case 475: 		/* #define ERR_BADCHANNELKEY    475 */
+	case 476:		/* #define ERR_BADCHANMASK      476 */
+	case 477:		/* #define ERR_NEEDREGGEDNICK	477 */
+	{
+		char *reason;
+
+		if (ArgList[0])
+			cant_join_channel(ArgList[0], from_server);
+
+		PasteArgs(ArgList, 0);
+		switch(comm)
+		{
+		    case 471:
+			reason = "Channel is full";
+			break;
+		    case 473:
+			reason = "You must be invited";
+			break;
+		    case 474:
+			reason = "You are banned";
+			break;
+		    case 475:
+			reason = "You must give the correct key";
+			break;
+		    case 476:
+			reason = "Bad channel mask";
+			break;
+		    case 477:
+			reason = "You must use a registered nickname";
+			break;
+		    default:
+			reason = "Because the server said so";
+			break;
+		}	
+
+		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+			put_it("%s %s (%s)", numeric_banner(), ArgList[0], reason);
+		break;
+	}
+
+
 	/*
 	 * The following accumulates the remaining arguments
 	 * in ArgSpace for hook detection. We can't use
@@ -1013,40 +1054,6 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 			break;
 		}
 
-		case 471:		/* #define ERR_CHANNELISFULL    471 */
-		case 473:		/* #define ERR_INVITEONLYCHAN   473 */
-		case 474:		/* #define ERR_BANNEDFROMCHAN   474 */
-		case 475: 		/* #define ERR_BADCHANNELKEY    475 */
-		case 476:		/* #define ERR_BADCHANMASK      476 */
-		{
-			char buffer[BIG_BUFFER_SIZE + 1];
-
-			if (ArgList[0])
-				remove_channel(ArgList[0], from_server);
-			PasteArgs(ArgList, 0);
-			strmcpy(buffer, ArgList[0], BIG_BUFFER_SIZE);
-			switch(comm)
-			{
-				case 471:
-					strmcat(buffer, " (Channel is full)", BIG_BUFFER_SIZE);
-					break;
-				case 473:
-					strmcat(buffer, " (You must be invited)", BIG_BUFFER_SIZE);
-					break;
-				case 474:
-					strmcat(buffer, " (You are banned)", BIG_BUFFER_SIZE);
-					break;
-				case 475:
-					strmcat(buffer, " (You must give the correct key)", BIG_BUFFER_SIZE);
-					break;
-				case 476:
-					strmcat(buffer, " (Bad channel mask)", BIG_BUFFER_SIZE);
-					break;
-			}
-			put_it("%s %s", numeric_banner(), buffer);
-			break;
-		}
-
 #define RPL_CLOSEEND         363
 		case 323:               /* #define RPL_LISTEND          323 */
 			funny_print_widelist();
