@@ -40,7 +40,7 @@
 const 	char *	umodes = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 static	char *	do_umode (int du_index);
 	void 	reset_nickname (int);
-static void	clear_reconnect_counts (void);
+	void	clear_reconnect_counts (void);
 
 	int	never_connected = 1;		/* true until first connection
 						 * is made */
@@ -111,7 +111,7 @@ void 	add_to_server_list (const char *server, int port, const char *password, co
 		s->resetting_nickname = 0;
 		s->reconnects = 0;
 		s->quit_message = NULL;
-		s->save_channels = 0;
+		s->save_channels = -1;
 
 		if (password && *password)
 			malloc_strcpy(&s->password, password);
@@ -534,8 +534,9 @@ char *	create_server_list (void)
 			if (server_list[i].itsname)
 				m_s3cat(&buffer, space, server_list[i].itsname);
 			else
-				yell("Warning: server_list[%d].itsname is "
-					"null and it shouldnt be", i);
+				yell("Warning: I don't have server #%d's real"
+					"name yet -- using the hostname you "
+					"gave me instead", i);
 		}
 	}
 
@@ -676,11 +677,17 @@ BUILT_IN_COMMAND(servercmd)
 	{
 		int	j = from_server;
 
-		clear_reconnect_counts();
 		i = find_server_refnum(server, &args);
-		server_reconnects_to(j, i);
-		reconnect(j);
-		window_check_servers();
+		if (i != j)
+		{
+			clear_reconnect_counts();
+			server_reconnects_to(j, i);
+			reconnect(j);
+			window_check_servers();
+		}
+		else
+			say("Connected to port %d of server %s",
+				server_list[j].port, server_list[j].name);
 	}
 }
 
@@ -706,6 +713,7 @@ void	do_server (fd_set *rd)
 		des = server_list[i].des;
 		if (des == -1 || !FD_ISSET(des, rd))
 			continue;
+		FD_CLR(des, rd);	/* Make sure it never comes up again */
 
 		last_server = from_server = i;
 		switch ((junk = dgets(bufptr, des, 1)))
@@ -715,6 +723,9 @@ void	do_server (fd_set *rd)
 
 			case -1:	/* EOF or other error */
 			{
+				if (server_list[i].save_channels == -1)
+					save_server_channels(i);
+
 				server_is_connected(i, 0);
 				close_server(i, NULL);
 				say("Connection closed from %s: %s", 
@@ -790,6 +801,7 @@ void 	flush_server (void)
 			{
 				if (FD_ISSET(des, &rd))
 				{
+					FD_CLR(des, &rd);
 					if (dgets(buffer, des, 0) == 0)
 						flushing = 0;
 				}
@@ -990,7 +1002,7 @@ static int 	connect_to_server (int new_server)
 	/*
 	 * Reset everything and go on our way.
 	 */
-	message_from((char *) 0, LOG_CRAP);
+	message_from(NULL, LOG_CRAP);
 	update_all_status();
 	server_reconnects_to(new_server, new_server + 1);
 	return 0;			/* New connection established */
@@ -1072,15 +1084,12 @@ int 	connect_to_new_server (int new_server, int old_server, int new_conn)
 	 */
 	if (new_conn == 0)
 	{
+		/* 
+		 * By default, all server channels are saved; however,
+		 * we save them just in case.
+		 */
 		if (is_server_open(old_server))
 		{
-			/*
-			 * We need to close off the old connection now.  
-			 * We make sure to set the "save_channels" hint, 
-			 * that way the channels wont be trashcanned, and 
-			 * the windows wont be re-assigned on us.  This 
-			 * gives us a chance to do the right thing below.
-			 */
 			save_server_channels(old_server);
 			close_server(old_server, "changing servers");
 		}
@@ -1189,6 +1198,9 @@ int	reconnect (int oldserv)
 
 		if (newserv != -1)
 			save_server_channels(oldserv);
+		else
+			dont_save_server_channels(oldserv);
+
 		if (newserv == oldserv || newserv == -1)
 			close_server(oldserv, get_server_quit_message(oldserv));
 		connected = is_server_connected(oldserv);
@@ -1200,7 +1212,10 @@ int	reconnect (int oldserv)
 	}
 
 	if (newserv < 0)
+	{
+		window_check_servers();
 		return -1;		/* User wants to disconnect */
+	}
 
 	/* Try all of the other servers, stop when one of them works. */
 	for (i = 0; i < number_of_servers; i++)
@@ -1241,6 +1256,7 @@ int	reconnect (int oldserv)
 		say("A new server connection could not be established.");
 		say("Your previous server connection will be resumed.");
 		from_server = oldserv;
+		window_check_servers();
 		return -1;
 	}
 
@@ -1258,6 +1274,8 @@ int	reconnect (int oldserv)
 	if (do_hook(DISCONNECT_LIST, "Unable to connect to a server"))
 		say("Sorry, cannot connect.  Use /SERVER to connect "
 						"to a server");
+
+	window_check_servers();
 	return -1;
 }
 
@@ -1302,12 +1320,15 @@ void	close_server (int old, const char *message)
 	if (waiting_out > waiting_in)		/* XXX - hack! */
 		waiting_out = waiting_in = 0;
 
-	if (server_list[old].save_channels)
+	if (server_list[old].save_channels == 1)
 		save_channels(old);
-	else
+	else if (server_list[old].save_channels == 0)
 		destroy_server_channels(old);
+	else
+		panic("Somebody forgot to set "
+			"server_list[%d].save_channels!", old);
 
-	server_list[old].save_channels = 0;
+	server_list[old].save_channels = -1;
 	server_list[old].oper = 0;
 	server_list[old].registration_pending = 0;
 	server_list[old].connected = 0;
@@ -1346,7 +1367,10 @@ void 	save_server_channels (int servnum)
 	server_list[servnum].save_channels = 1;
 }
 
-
+void	dont_save_server_channels (int servnum)
+{
+	server_list[servnum].save_channels = 0;
+}
 
 
 /********************* OTHER STUFF ************************************/
@@ -1607,8 +1631,10 @@ void 	password_sendline (char *data, char *line)
 
 	if (line && *line)
 	{
-		new_server = atoi(next_arg(line, &line));
+		new_server = atoi(data);
 		set_server_password(new_server, line);
+		change_window_server(new_server, new_server);
+		server_reconnects_to(new_server, new_server);
 		reconnect(new_server);
 	}
 }
@@ -1667,6 +1693,13 @@ void 	server_is_connected (int sic_index, int value)
 	server_list[sic_index].rejoined_channels = 0;
 	if (value)
 	{
+		/* 
+		 * By default, we want to save the server's channels.
+		 * If anything happens where we don't want to do this,
+		 * then we must turn it off, rather than the other way
+		 * around.
+		 */
+		save_server_channels(sic_index);
 		server_list[sic_index].reconnect_to = sic_index;
 		server_list[sic_index].eof = 0;
 		clear_reconnect_counts();
@@ -1714,8 +1747,10 @@ BUILT_IN_COMMAND(disconnectcmd)
 			message = args;
 
 		say("Disconnecting from server %s", get_server_itsname(i));
-		send_to_server("QUIT :%s", message);
 		server_reconnects_to(i, -1);
+		dont_save_server_channels(i);
+		close_server(i, message);
+		update_all_status();
 	}
 
 	if (!connected_to_server)
@@ -2004,6 +2039,7 @@ void 	fudge_nickname (int servnum)
 	 */
 	if (s->nickname_pending)
 	{
+		nick_command_is_pending(servnum, 0);
 		new_free(&s->s_nickname);
 		return;
 	}
@@ -2152,7 +2188,7 @@ void 	save_servers (FILE *fp)
 	}
 }
 
-static void	clear_reconnect_counts (void)
+void	clear_reconnect_counts (void)
 {
 	int	i;
 
